@@ -229,7 +229,6 @@ job_finished_cb (EvJob         *job,
 	job_info = find_job_cache (pixbuf_cache, job_render->rc->page);
 
 	copy_job_to_job_info (job_render, job_info, pixbuf_cache);
-
 	g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0);
 }
 
@@ -449,6 +448,7 @@ copy_job_to_job_info (EvJobRender   *job_render,
 		job_info->selection_region = gdk_region_copy (job_render->selection_region);
 		job_info->selection = g_object_ref (job_render->selection);
 		g_assert (job_info->selection_points.x1 >= 0);
+		job_info->points_set = TRUE;
 	}
 
 	if (job_info->job) {
@@ -681,6 +681,7 @@ ev_pixbuf_cache_get_pixbuf (EvPixbufCache *pixbuf_cache,
 	if (job_info->job &&
 	    EV_JOB (job_info->job)->finished) {
 		copy_job_to_job_info (EV_JOB_RENDER (job_info->job), job_info, pixbuf_cache);
+		g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0);
 	}
 
 	return job_info->pixbuf;
@@ -700,6 +701,7 @@ ev_pixbuf_cache_get_link_mapping (EvPixbufCache *pixbuf_cache,
 	if (job_info->job &&
 	    EV_JOB (job_info->job)->finished) {
 		copy_job_to_job_info (EV_JOB_RENDER (job_info->job), job_info, pixbuf_cache);
+		g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0);
 	}
 
 	return job_info->link_mapping;
@@ -722,6 +724,7 @@ ev_pixbuf_cache_get_image_mapping (EvPixbufCache *pixbuf_cache,
 	if (job_info->job &&
 	    EV_JOB (job_info->job)->finished) {
 		copy_job_to_job_info (EV_JOB_RENDER (job_info->job), job_info, pixbuf_cache);
+		g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0);
 	}
 
 	return job_info->image_mapping;
@@ -779,6 +782,7 @@ ev_pixbuf_cache_get_text_mapping (EvPixbufCache *pixbuf_cache,
 	if (job_info->job &&
 	    EV_JOB (job_info->job)->finished) {
 		copy_job_to_job_info (EV_JOB_RENDER (job_info->job), job_info, pixbuf_cache);
+		g_signal_emit (pixbuf_cache, signals[JOB_FINISHED], 0);
 	}
 	
 	return job_info->text_mapping;
