@@ -220,6 +220,20 @@ clear_range (EvSidebarThumbnails *sidebar_thumbnails,
 	gtk_tree_path_free (path);
 }
 
+static gdouble
+get_scale_for_page (EvSidebarThumbnails *sidebar_thumbnails,
+		    gint                 page)
+{
+	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
+	gint width, height;
+
+	ev_page_cache_get_size (priv->page_cache,
+				page, priv->rotation,
+				1.0, &width, &height);
+	
+	return (gdouble)THUMBNAIL_WIDTH / (gdouble)width;
+}
+
 static void
 add_range (EvSidebarThumbnails *sidebar_thumbnails,
 	   gint                 start_page,
@@ -237,7 +251,7 @@ add_range (EvSidebarThumbnails *sidebar_thumbnails,
 	for (result = gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->list_store), &iter, path);
 	     result && page <= end_page;
 	     result = gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->list_store), &iter), page ++) {
-		EvJobThumbnail *job;
+		EvJob *job;
 		gboolean thumbnail_set;
 
 		gtk_tree_model_get (GTK_TREE_MODEL (priv->list_store), &iter,
@@ -246,9 +260,14 @@ add_range (EvSidebarThumbnails *sidebar_thumbnails,
 				    -1);
 
 		if (job == NULL && !thumbnail_set) {
-			/* FIXME: Need rotation */
-			job = (EvJobThumbnail *)ev_job_thumbnail_new (priv->document, page, priv->rotation, THUMBNAIL_WIDTH);
+			EvRenderContext *rc;
+
+			rc = ev_render_context_new (priv->rotation, page,
+						    get_scale_for_page (sidebar_thumbnails, page));
+			job = ev_job_thumbnail_new (priv->document, rc);
 			ev_job_queue_add_job (EV_JOB (job), EV_JOB_PRIORITY_HIGH);
+			g_object_unref (rc);
+			
 			g_object_set_data_full (G_OBJECT (job), "tree_iter",
 						gtk_tree_iter_copy (&iter),
 						(GDestroyNotify) gtk_tree_iter_free);
@@ -258,6 +277,7 @@ add_range (EvSidebarThumbnails *sidebar_thumbnails,
 			gtk_list_store_set (priv->list_store, &iter,
 					    COLUMN_JOB, job,
 					    -1);
+			
 			/* The queue and the list own a ref to the job now */
 			g_object_unref (job);
 		} else if (job) {
@@ -334,14 +354,11 @@ adjustment_changed_cb (EvSidebarThumbnails *sidebar_thumbnails)
 		return;
 	}
 
-	if (path == NULL)
-		path = gtk_tree_path_new_first ();
-	if (path2 == NULL)
-		path2 = gtk_tree_path_new_from_indices (priv->n_pages,
-							-1);
-	update_visible_range (sidebar_thumbnails,
-			      gtk_tree_path_get_indices (path)[0],
-			      gtk_tree_path_get_indices (path2)[0]);
+	if (path && path2) {
+		update_visible_range (sidebar_thumbnails,
+				      gtk_tree_path_get_indices (path)[0],
+				      gtk_tree_path_get_indices (path2)[0]);
+	}
 
 	gtk_tree_path_free (path);
 	gtk_tree_path_free (path2);
@@ -376,26 +393,37 @@ ev_sidebar_thumbnails_fill_model (EvSidebarThumbnails *sidebar_thumbnails)
 static void
 ev_sidebar_thumbnails_set_loading_icon (EvSidebarThumbnails *sidebar_thumbnails)
 {
-	gint width = THUMBNAIL_WIDTH;
-	gint height = THUMBNAIL_WIDTH;
-
 	if (sidebar_thumbnails->priv->loading_icon)
 		g_object_unref (sidebar_thumbnails->priv->loading_icon);
 
 	if (sidebar_thumbnails->priv->document) {
-		/* We get the dimensions of the first doc so that we can make a blank
+		gint width = THUMBNAIL_WIDTH;
+		gint height;
+		gint page_width, page_height;
+
+		/* We get the dimensions of the first page so that we can make a blank
 		 * icon.  */
-		ev_document_doc_mutex_lock ();
-		ev_document_thumbnails_get_dimensions (EV_DOCUMENT_THUMBNAILS (sidebar_thumbnails->priv->document),
-						       0, THUMBNAIL_WIDTH, &width, &height);
-		ev_document_doc_mutex_unlock ();
+		ev_page_cache_get_size (sidebar_thumbnails->priv->page_cache, 0,
+					sidebar_thumbnails->priv->rotation,
+					1.0, &page_width, &page_height);
+
+		height = (gint) (page_height * ((gdouble)width / page_width));
+		
 		sidebar_thumbnails->priv->loading_icon =
-			ev_document_misc_get_thumbnail_frame (width, height, sidebar_thumbnails->priv->rotation, NULL);
+			ev_document_misc_get_thumbnail_frame (width, height, NULL);
 	} else {
 		sidebar_thumbnails->priv->loading_icon = NULL;
 	}
 
 }
+
+static gboolean
+refresh (EvSidebarThumbnails *sidebar_thumbnails)
+{
+	adjustment_changed_cb (sidebar_thumbnails);
+	return FALSE;
+}
+
 void
 ev_sidebar_thumbnails_refresh (EvSidebarThumbnails *sidebar_thumbnails,
 			       int                  rotation)
@@ -412,7 +440,7 @@ ev_sidebar_thumbnails_refresh (EvSidebarThumbnails *sidebar_thumbnails,
 	/* Trigger a redraw */
 	sidebar_thumbnails->priv->start_page = 0;
 	sidebar_thumbnails->priv->end_page = 0;
-	adjustment_changed_cb (sidebar_thumbnails);
+	g_idle_add ((GSourceFunc)refresh, sidebar_thumbnails);
 }
 
 static void
@@ -432,7 +460,7 @@ ev_sidebar_tree_selection_changed (GtkTreeSelection *selection,
 	page = gtk_tree_path_get_indices (path)[0];
 	gtk_tree_path_free (path);
 
-	ev_page_cache_set_current_page (priv->page_cache, page);
+	ev_page_cache_set_current_page_history (priv->page_cache, page);
 }
 
 static void
@@ -457,7 +485,7 @@ ev_sidebar_icon_selection_changed (GtkIconView         *icon_view,
 	gtk_tree_path_free (path);
 	g_list_free (selected);
 
-	ev_page_cache_set_current_page (priv->page_cache, page);
+	ev_page_cache_set_current_page_history (priv->page_cache, page);
 }
 
 static void
@@ -562,7 +590,17 @@ page_changed_cb (EvPageCache         *page_cache,
 		gtk_tree_view_set_cursor (tree_view, path, NULL, FALSE);
 		gtk_tree_view_scroll_to_cell (tree_view, path, NULL, FALSE, 0.0, 0.0);
 	} else if (sidebar->priv->icon_view) {
+
+		g_signal_handlers_block_by_func
+			(sidebar->priv->icon_view,
+			 G_CALLBACK (ev_sidebar_icon_selection_changed), sidebar);
+
 		gtk_icon_view_select_path (GTK_ICON_VIEW (sidebar->priv->icon_view), path);
+
+		g_signal_handlers_unblock_by_func
+			(sidebar->priv->icon_view,
+			 G_CALLBACK (ev_sidebar_icon_selection_changed), sidebar);
+
 		gtk_icon_view_set_cursor (GTK_ICON_VIEW (sidebar->priv->icon_view), path, NULL, FALSE);
 	}
 
@@ -633,6 +671,9 @@ ev_sidebar_thumbnails_set_document (EvSidebarPage	*sidebar_page,
 	g_signal_connect (priv->page_cache, "page-changed", G_CALLBACK (page_changed_cb), sidebar_thumbnails);
 	sidebar_thumbnails->priv->start_page = 0;
 	sidebar_thumbnails->priv->end_page = 0;
+	page_changed_cb (priv->page_cache,
+			 ev_page_cache_get_current_page (priv->page_cache),
+			 sidebar_thumbnails);
 	adjustment_changed_cb (sidebar_thumbnails);
 }
 
